#!/bin/sh

mkdir -p web/static
mkdir -p gem
mkdir -p tmp


fecha=$(date +%F)

echo "empezando conversión, estamos en ${fecha}"

echo "revisando y copiando imágenes..."
for f in $(find src/ -regex ".*\(jpg\|png\|gif\|svg\)")
do
	path="web/${f#src/}" # quita el prefijo de src/ y agrega web/
	gempath="gem/${f#src/}" # quita el prefijo de src/ y agrega gem/
	mkdir -p $(dirname $path) # crea directorio si no existe
	mkdir -p $(dirname $gempath) # crea directorio si no existe
	cp -vu $f $path
	cp -vu $f $gempath
done

# limpia
echo "limpiando archivos previos..."

rm web/*html
rm gem/*gmi
rm tmp/*gmo

# genera logs
awk -f genlog.awk src/log.txt

# copia
echo "copiando archivos..."

#cp src/llave_sejo.asc web/
#cp src/llave_sejo.asc gem/

cp src/estilo.css web/static/


# crea índice
echo "generando índice..."
rm src/pages.gmo
ls -t src/*gmo | awk -f genindice.awk > src/pages.gmo

# genera archivos con incoming links en tmp/
echo "generando incoming links..."
python3 links.py

echo "revisando y convirtiendo archivos gmo a html y gmi..."

i=0
# convierte y actualiza archivos gmi a html
for f in $(find tmp/ -iname *.gmo)
do

	path=${f#tmp/} # quita el prefijo de "tmp/"
	htmlpath="web/${path%gmo}html" # agrega "web/" y cambia el sufijo
	gempath="gem/${path%gmo}gmi" # agrega "gem/"

	fecha=$(date -r src/$path +%F)

	# haz la conversión
	awk -v fecha=$fecha -f gemtext2html.awk $f > $htmlpath
	awk -v fecha=$fecha -f gmo2gmi.awk $f   > $gempath

	i=$((i+1))

done

cp web/home.html web/index.html
cp gem/home.gmi gem/index.gmi

echo "convertidas ${i} páginas!"
